/*global tizen, document, window, SAPConnection, Image, setTimeout*/
/*
* Copyright (c) 2015 Samsung Electronics Co., Ltd.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
* * Redistributions in binary form must reproduce the above
* copyright notice, this list of conditions and the following disclaimer
* in the documentation and/or other materials provided with the
* distribution.
* * Neither the name of Samsung Electronics Co., Ltd. nor the names of its
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
var popup = tau.widget.popup(document.getElementById("popupToast")),
    popupContent = document.getElementById("popupContent"),
    popupTimeout = 0,

    images = [],
    offset = 0,

    LIST_LIMIT = 3,
    POPUP_TIME = 2000;

/**
 * Set info message in DOM
 * @param {String} msg
 */
function setMessage(msg, autoHide) {
    'use strict';
    clearTimeout(popupTimeout);
 
    popupContent.innerHTML = msg;

    if (!popup._isOpened()) {
        tau.openPopup("#popupToast");
    }

    if (autoHide) {
        popupTimeout = setTimeout(tau.closePopup.bind(tau), POPUP_TIME);
    }
}

/**
 * Resize and convert image to base64
 * @param {Object} image
 * @param {Number} width
 * @return {String}
 */
function imageToBase64(image, width) {
    'use strict';
    var canvas = document.createElement('CANVAS'),
        ctx = canvas.getContext('2d');

    if (image.width > width) {
        image.height *= width / image.width;
        image.width = width;
    }

    canvas.width = image.width;
    canvas.height = image.height;
    ctx.drawImage(image, 0, 0, image.width, image.height);

    return canvas.toDataURL().replace(/(.*)base64\,/, '');
}

/**
 * Prepare image object
 * @param {Number} index
 * @param {Number} width
 * @return {Object}
 */
function getImage(index, width) {
    'use strict';
    var cur = images[index],
        base64 = imageToBase64(cur.image, width);

    return {
        id: cur.id,
        name: cur.content.name,
        height: cur.image.height * width / cur.image.width,
        width: width,
        size: base64.length,
        image: base64
    };
}

/**
 * Prepare thumbnails list
 * @return {Array}
 */
function getThumbnails() {
    'use strict';
    var result = [],
        i = 0;

    for (i = offset; i < images.length; i += 1) {
        result.push(getImage(i, 120));

        if (result.length === LIST_LIMIT) {
            break;
        }
    }

    return result;
}

/**
 * Handler for SAP connection request
 * @param {Number} channelId
 * @param {Object} SASocket
 * @param {Object} data
 */
function onRequest(channelId, SASocket, data) {
    'use strict';
    var response = {
        result: 'success',
        reason: 1,
        count: images.length
    };

    switch (data.msgId) {
    case 'gallery-image-req':
        response.msgId = 'gallery-image-rsp';
        response.image = getImage(data.id, data.width);

        SASocket.sendData(channelId, JSON.stringify(response));
        setMessage('Image have been sent', true);
        break;
    case 'gallery-thumbnail-req':
        if (data.offset === -1) {
            offset = 0;
        } else {
            offset += 1;
        }

        response.msgId = 'gallery-thumbnail-rsp';
        response.list = getThumbnails();

        SASocket.sendData(channelId, JSON.stringify(response));
        setMessage('Thumbnails have been sent', true);
        break;
    }
}

/**
 * Initialize SAP connection
 */
function readyToCommunicate() {
    'use strict';

    var SAPConn = new SAPConnection(
        'GalleryConsumer',
        'PROVIDER',
        onRequest,
        setMessage
    );

    SAPConn.connect();
}

/**
 * Error callback on content find
 */
function onLoadImagesError() {
    'use strict';
    setMessage('Cannot load images');
}

/**
 * Prepare and load images to array
 */
function prepareImages(files, index, callback) {
    'use strict';
    var image = new Image();

    image.onload = function () {
        images.push({
            id: index,
            content: files[index],
            image: image
        });

        if (files[++index]) {
            prepareImages(files, index, callback);
        } else {
            callback();
        }
    };

    image.src = files[index].contentURI;
}

/**
 * Success callback on content find
 * @param {Array} files
 */
function onLoadImagesSuccess(files) {
    'use strict';

    if (files.length === 0) {
        setMessage('No images found');
    } else {
        setMessage('Loading images...<br/>Please wait');
        prepareImages(files, 0, readyToCommunicate);
    }
}

/**
 * Load images file using Content API
 */
function loadImages() {
    'use strict';
    var filter = new tizen.AttributeFilter('type', 'EXACTLY', 'IMAGE');

    tizen.content.find(onLoadImagesSuccess, onLoadImagesError, null, filter);
}

/**
 * On window load
 */
function onWindowLoad() {
    'use strict';
    setTimeout(tau.openPopup.bind(tau, "#popupToast"));

    loadImages();

    document.addEventListener('tizenhwkey', function (e) {
        if (e.keyName === 'back') {
            tizen.application.getCurrentApplication().exit();
        }
    });

}

window.onload = onWindowLoad;
